@@ -10,7 +10,7 @@
 
 #include <coreplugin/editormanager/editormanager.h>
 #include <texteditor/codeassist/iassistproposal.h>
-#include <texteditor/syntaxhighlighterrunner.h>
+#include <texteditor/syntaxhighlighter.h>
 #include <texteditor/textdocument.h>
 #include <texteditor/texteditor.h>
 
@@ -118,7 +118,7 @@ public:
 
         const int pos = proposal->basePosition();
         const int length = m_position - pos;
-        const QString prefix = Utils::Text::textAt(QTextCursor(m_textDocument), pos, length);
+        const QString prefix = Utils::Text::textAt(m_textDocument, pos, length);
         if (!prefix.isEmpty())
             listmodel->filter(prefix);
         if (listmodel->isSortable(prefix))
@@ -146,12 +146,10 @@ public:
         TextEditor::BaseTextEditor *cppEditor = qobject_cast<TextEditor::BaseTextEditor *>(m_editor);
         if (!cppEditor)
             return false;
-        if (cppEditor->textDocument()->syntaxHighlighterRunner()->syntaxInfoUpdated())
+        if (cppEditor->textDocument()->syntaxHighlighter()->syntaxHighlighterUpToDate())
             return true;
         return ::CppEditor::Tests::waitForSignalOrTimeout(
-            cppEditor->textDocument()->syntaxHighlighterRunner(),
-            &SyntaxHighlighterRunner::highlightingFinished,
-            5000);
+            cppEditor->textDocument()->syntaxHighlighter(), &SyntaxHighlighter::finished, 5000);
     }
 
 private:
@@ -2227,7 +2225,7 @@ void CompletionTest::testCompletion_data()
 
     QTest::newRow("no_binding_block_as_instantiationOrigin_QTCREATORBUG-11424") << _(
             "template <typename T>\n"
-            "class QVector\n"
+            "class QList\n"
             "{\n"
             "public:\n"
             "   inline const_iterator constBegin() const;\n"
@@ -2235,8 +2233,8 @@ void CompletionTest::testCompletion_data()
             "\n"
             "typedef struct { double value; } V;\n"
             "\n"
-            "double getValue(const QVector<V>& d) const {\n"
-            "   typedef QVector<V>::ConstIterator Iter;\n"
+            "double getValue(const QList<V>& d) const {\n"
+            "   typedef QList<V>::ConstIterator Iter;\n"
             "   @\n"
             "}\n"
         ) << _("double val = d.constBegin()->") << QStringList();
@@ -2891,6 +2889,16 @@ void CompletionTest::testCompletionMemberAccessOperator_data()
         ) << _("p->") << QStringList({"S", "m"})
         << false
         << false;
+    QTest::newRow("dot to arrow: template + reference + double typedef")
+        << _("template <typename T> struct C {\n"
+             "    using ref = T &;\n"
+             "    ref operator[](int i);\n"
+             "};\n"
+             "struct S { int m; };\n"
+             "template<typename T> using CS = C<T>;\n"
+             "CS<S *> v;\n"
+             "@\n")
+        << _("v[0].") << QStringList({"S", "m"}) << false << true;
 }
 
 } // namespace CppEditor::Internal
