@@ -187,7 +187,9 @@ QString Toolchain::detectionSource() const
 
 ToolchainFactory *Toolchain::factory() const
 {
-    return ToolchainFactory::factoryForType(typeId());
+    ToolchainFactory * const factory = ToolchainFactory::factoryForType(typeId());
+    QTC_ASSERT(factory, qDebug() << typeId());
+    return factory;
 }
 
 QByteArray Toolchain::id() const
@@ -209,6 +211,7 @@ bool Toolchain::canShareBundle(const Toolchain &other) const
 {
     QTC_ASSERT(typeId() == other.typeId(), return false);
     QTC_ASSERT(language() != other.language(), return false);
+    QTC_ASSERT(factory(), return false);
 
     if (int(factory()->supportedLanguages().size()) == 1)
         return false;
@@ -709,7 +712,7 @@ Toolchain *ToolchainFactory::restore(const Store &data)
     QTC_ASSERT(tc, return nullptr);
 
     tc->fromMap(data);
-    if (!tc->hasError())
+    if (!tc->hasError() && QTC_GUARD(tc->typeId() == supportedToolchainType()))
         return tc;
 
     delete tc;
@@ -741,12 +744,11 @@ void ToolchainFactory::autoDetectionToMap(Store &data, bool detected)
 
 Toolchain *ToolchainFactory::createToolchain(Id toolchainType)
 {
-    for (ToolchainFactory *factory : std::as_const(toolchainFactories())) {
-        if (factory->m_supportedToolchainType == toolchainType) {
-            if (Toolchain *tc = factory->create()) {
-                tc->d->m_typeId = toolchainType;
-                return tc;
-            }
+    if (ToolchainFactory * const factory = factoryForType(toolchainType)) {
+        if (Toolchain * const tc = factory->create()) {
+            QTC_ASSERT(tc->typeId() == toolchainType, qDebug() << toolchainType.toSetting());
+            tc->d->m_typeId = toolchainType; // FIXME: Redundant?
+            return tc;
         }
     }
     return nullptr;
@@ -920,6 +922,7 @@ ToolchainBundle::ToolchainBundle(const Toolchains &toolchains, HandleMissing han
 {
     // Check pre-conditions.
     QTC_ASSERT(!m_toolchains.isEmpty(), return);
+    QTC_ASSERT(factory(), return);
     QTC_ASSERT(m_toolchains.size() <= factory()->supportedLanguages().size(), return);
     for (const Toolchain * const tc : toolchains) {
         QTC_ASSERT(factory()->supportedLanguages().contains(tc->language()), return);
@@ -935,8 +938,9 @@ ToolchainBundle::ToolchainBundle(const Toolchains &toolchains, HandleMissing han
     // Check post-conditions.
     QTC_ASSERT(m_toolchains.size() == m_toolchains.first()->factory()->supportedLanguages().size(),
                return);
-    for (auto i = toolchains.size(); i < m_toolchains.size(); ++i)
+    for (auto i = toolchains.size(); i < m_toolchains.size(); ++i) {
         QTC_ASSERT(m_toolchains.at(i)->typeId() == m_toolchains.first()->typeId(), return);
+    }
 
     Utils::sort(m_toolchains, [](const Toolchain *tc1, const Toolchain *tc2) {
         return tc1 != tc2 && tc1->language() == Constants::C_LANGUAGE_ID;
@@ -962,7 +966,7 @@ QList<ToolchainBundle> ToolchainBundle::collectBundles(
         toolchainsPerBundleId.erase(unbundled);
     }
 
-    for (const Toolchains &tcs : toolchainsPerBundleId)
+    for (const Toolchains &tcs : std::as_const(toolchainsPerBundleId))
         bundles.emplaceBack(tcs, handleMissing);
     return bundles;
 }
@@ -1035,7 +1039,7 @@ QList<ToolchainBundle> ToolchainBundle::bundleUnbundledToolchains(const Toolchai
             if (nextBundle.isEmpty())
                 break;
             const Id newBundleId = Id::generate();
-            for (Toolchain * const tc : nextBundle)
+            for (Toolchain * const tc : std::as_const(nextBundle))
                 tc->setBundleId(newBundleId);
             bundles.emplaceBack(nextBundle, handleMissing);
         }
